"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenAIAssistant = void 0;
const openai_1 = require("openai");
const log_1 = require("@brainstack/log");
const hub_1 = require("@brainstack/hub");
class OpenAIAssistant {
    constructor(options, assistantId) {
        this.openai = new openai_1.OpenAI(options);
        this.thread = null;
        this.logger = (0, log_1.createLogger)();
        this.status = 'idle';
        this.assistantId = assistantId;
        this.functions = {};
        this.hub = (0, hub_1.createEventHub)();
    }
    on(_eventName, _callback) {
        this.hub.on(_eventName, _callback);
    }
    getStatus() {
        return this.status;
    }
    addTool(name, func) {
        this.functions[name] = func;
    }
    removeTool(name) {
        if (this.functions[name]) {
            delete this.functions[name];
        }
    }
    initThread() {
        return __awaiter(this, void 0, void 0, function* () {
            this.thread = yield this.openai.beta.threads.create();
        });
    }
    pollAnswers(threadId, runId) {
        return __awaiter(this, void 0, void 0, function* () {
            let completed = false;
            while (!completed) {
                try {
                    const response = yield this.openai.beta.threads.runs.retrieve(threadId, runId);
                    this.updateStatus(response.status);
                    if (this.isRunCompleted(response.status)) {
                        this.logger.info('Run completed!');
                        yield this.handleCompletedRun(threadId);
                        this.hub.emit('complete', { threadId });
                        completed = true;
                    }
                    else if (response.status === 'requires_action' &&
                        response['required_action']) {
                        this.logger.info('Run Action Required!');
                        this.hub.emit('required_action', {
                            threadId,
                            runId,
                            required_action: response['required_action'],
                        });
                        yield this.handleActionRequired(threadId, runId, response['required_action']);
                    }
                    else if (this.isRunFailedOrCancelled(response.status)) {
                        this.logger.info(`Completed ${response.status}`);
                        this.hub.emit('failed_canceled', {
                            response,
                        });
                        completed = true;
                    }
                    else if (this.isRunInProgress(response.status)) {
                        this.logger.info(`Run ${response.status}`);
                        this.hub.emit('in_progress');
                        yield this.delay(2000); // Poll every 2 seconds
                    }
                }
                catch (error) {
                    this.logger.error('Error in polling (retrying):', error);
                }
            }
        });
    }
    updateStatus(newStatus) {
        this.status = newStatus;
        this.hub.emit('status_changed', {
            status: newStatus,
        });
    }
    isRunCompleted(status) {
        return status === 'completed' || status === 'expired';
    }
    isRunFailedOrCancelled(status) {
        return ['failed', 'cancelled', 'expired', 'cancelling'].includes(status);
    }
    isRunInProgress(status) {
        return status === 'in_progress' || status === 'queued';
    }
    handleActionRequired(threadId, runId, requiredAction) {
        return __awaiter(this, void 0, void 0, function* () {
            const tool_outputs = yield this.processRequiredActions(requiredAction);
            yield this.openai.beta.threads.runs.submitToolOutputs(threadId, runId, {
                tool_outputs,
            });
        });
    }
    processRequiredActions(requiredAction) {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.all(requiredAction['submit_tool_outputs']['tool_calls'].map((ra) => __awaiter(this, void 0, void 0, function* () {
                if (ra.type === 'function' && ra.function.name in this.functions) {
                    this.logger.info(`Running ${ra.function.name} with arguments ${ra.function.arguments}`);
                    const ret = yield Promise.resolve(this.functions[ra.function.name](ra.function.arguments));
                    this.logger.info('Returned ', ret);
                    return {
                        tool_call_id: ra.id,
                        output: JSON.stringify(ret),
                    };
                }
                return {
                    tool_call_id: ra.id,
                    output: 'There is a problem, function not found!',
                };
            })));
        });
    }
    handleCompletedRun(threadId) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const respmsg = yield this.openai.beta.threads.messages.list(threadId);
            this.hub.emit('run_completed', {
                threadId,
                respmsg
            });
            if ((_a = respmsg.data[0].content[0]) === null || _a === void 0 ? void 0 : _a.hasOwnProperty('text')) {
                this.logger.info((_c = (_b = respmsg.data[0].content[0]) === null || _b === void 0 ? void 0 : _b.text) === null || _c === void 0 ? void 0 : _c.value);
            }
        });
    }
    addMessage(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const msg = yield this.openai.beta.threads.messages.create(this.thread.id, {
                role: 'user',
                content: message,
            });
        });
    }
    cancel(runId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.openai.beta.threads.runs.cancel(this.thread.id, runId);
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('RUN STEP: ' + this.assistantId);
            const response = yield this.openai.beta.threads.runs.create(this.thread.id, {
                assistant_id: this.assistantId,
            });
            yield this.pollAnswers(this.thread.id, response.id);
        });
    }
    delay(ms) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => setTimeout(resolve, ms));
        });
    }
}
exports.OpenAIAssistant = OpenAIAssistant;
